@@ -1,16 +1,16 @@
 #!/usr/bin/env python
 
 from pypepa.logger import init_log
+from pypepa.exceptions import DeadlockError
 
 class StateSpace():
-    operators = []
-    components = []
-    comp_ss = None
-
 
     def __init__(self):
         self.max_length = 0
         self.log = init_log()
+        self.operators = []
+        self.components = []
+        self.comp_ss = None
 
     def _combine_states(self, states):
         """ When more than one transition leads to the same state,
@@ -33,21 +33,70 @@ class StateSpace():
         states = [i for i in states if i is not None]
         return states
 
+    def derive(self, alg="BU", dotdir=None):
+        if alg == "BU":
+            return self.derive_bu(dotdir=dotdir)
+        elif alg == "TD":
+            return self.derive_td()
 
-    def derive(self):
-        """ Derives the whole state space according to BU"""
+
+    def derive_td(self):
+        raise Exception("Top down derivation currently unimplemented")
+
+    def _derive_state_from_one_component(self):
+        """ Derives state space with BU algorithm when system equation is just one 
+        component """
         initial_state = []
         queue = []
         visited = []
         resulting_states = {}
         actions_to_state = {}
         state_num = 0
-        for x in list(range(0,self.max_length+1,1)):
-            for op in self.operators:
-                if op.length == x:
-                    op.update_offset()
-        for comp in self.components:
-            initial_state.append(comp.name)
+        initial_state.append(self.components[0].name)
+        queue.append(initial_state)
+        while(queue):
+            state = queue.pop(0)
+            if self._gs_to_string(state) in visited:
+                continue
+            state_num = state_num + 1
+            self.log.debug("{} STATE {}".format(state_num, state))
+            resulting_states[self._gs_to_string(state)] = ([],state_num)
+            visited.append(self._gs_to_string(state))
+            self.components[0].update( state[0] )
+            new_states = []
+            for transition in self.components[0].get_derivatives():
+                new_states.append(transition)
+            if not new_states:
+                self.log.error("DEADLOCK in {}".format(state))
+                raise DeadlockError("Deadlock in state: {}".format(state))
+            new_states = self._combine_states(new_states[:])
+            for news in new_states:
+                self.log.debug("{}\trate={} \t-> {}".format(news.action, news.arate, news.to_s))
+                resulting_states[self._gs_to_string(state)][0].append( (news.rate, self._gs_to_string(news.to_s), news.action))
+                if news.combined:
+                    for act in news.actions:
+                        self._add_to_actions_set(act, news.actions[act], actions_to_state, state_num)
+                else:
+                    self._add_to_actions_set(news.action, news.rate, actions_to_state, state_num)
+
+                if self._gs_to_string(news.to_s) not in visited:
+                    queue.append(news.to_s)
+        return (resulting_states, actions_to_state)
+
+    def derive_bu(self,dotdir=None):
+        """ Derives the whole state space using Bottom-up algorithm""" 
+        initial_state = []
+        queue = []
+        visited = []
+        resulting_states = {}
+        actions_to_state = {}
+        state_num = 0
+        if len(self.operators) == 0:
+            self.log.debug("System equation has no operators")
+            return  self._derive_state_from_one_component()
+
+        [op.update_offset() for op in sorted(self.operators, key=lambda x: x.length)]
+        [initial_state.append(comp.name) for comp in self.components]
         queue.append(initial_state)
         while(queue):
             state = queue.pop(0)
@@ -62,6 +111,7 @@ class StateSpace():
             for i in list( range(0, len( state ),1)):
                 self.components[i].update( state[i] )
             # for every length can be changed
+
             for x in list(range(0,self.max_length+1,1)):
                 for op in self.operators:
                     if op.length == x:
@@ -72,8 +122,7 @@ class StateSpace():
                             new_states = op.compose(self.comp_ss, state, True)
                             if not new_states:
                                 self.log.error("DEADLOCK in {}".format(state))
-                                print("DEADLOCK in {}".format(state))
-                                exit(1)
+                                raise DeadlockError("Deadlock in state: {}".format(state))
                             new_states = self._combine_states(new_states[:])
                             for news in new_states:
                                 # aggregate
@@ -82,7 +131,7 @@ class StateSpace():
                                     new_state[news.offset] = news.to_s[0]
                                     news.to_s = new_state
                                 self.log.debug("{}\trate={} \t-> {}".format(news.action, news.arate, news.to_s))
-                                resulting_states[self._gs_to_string(state)][0].append( (news.rate, self._gs_to_string(news.to_s)))
+                                resulting_states[self._gs_to_string(state)][0].append( (news.rate, self._gs_to_string(news.to_s), news.action))
                                 #handle combines actions, not very elegant so to be changed
                                 if news.combined:
                                     for act in news.actions:
@@ -99,23 +148,36 @@ class StateSpace():
     def _add_to_actions_set(self, action, rate,actions_to_state, state_num):
         if rate == "infty":
             self.log.error("Resulting rate cannot be infty in {} in state {}".format(action, state_num))
-            exit(1)
+            raise ValueError("Resulting rate cannot be infty in {} in state {}".format(action, state_num))
         if (action,state_num) not in actions_to_state:
             actions_to_state[ (action, state_num) ] = float(rate)
         else:
-            #check if the same stateand action, so we add rates
+            #check if the same state and action, so we add rates
             actions_to_state[ (action, state_num) ] += float(rate)
 
     def _gs_to_string(self, gs_list):
         return ','.join( map( str, gs_list ) )
 
 class Component():
-    length = None
-    offset = None
-    name = None
-    ss = None
-    data = None
+    def __init__(self, ss, name,offset):
+        self.name = name
+        self.ss = ss
+        self.offset = offset
+        self.derivatives = []
+        arates = {}
+        for der in self.ss[self.name].transitions:
+            if der.action not in arates:
+                if der.rate  in ("infty", "T"):
+                    arates[der.action] = float(-1)
+                else:
+                    arates[der.action] = float(der.rate)
+            else:
+                arates[der.action] += float(der.rate)
 
+        for der in self.ss[self.name].transitions:
+            if der.rate in ("infty", "T"):
+                der.rate = -1
+            self.derivatives.append(Derivative(self.name, [der.to], der.action, der.rate, self.offset, der.rate, aprates=arates))
 
     def update(self, name):
         """
@@ -128,7 +190,7 @@ class Component():
         arates = {}
         for der in self.ss[self.name].transitions:
             if der.action not in arates:
-                if der.rate == "infty":
+                if der.rate in ("infty", "T"):
                     arates[der.action] = float(-1)
                 else:
                     arates[der.action] = float(der.rate)
@@ -136,32 +198,10 @@ class Component():
                 arates[der.action] += float(der.rate)
         for der in self.ss[self.name].transitions:
             # print("self.name:%s der.action:%s der.rate:%s de.to:%s" % (self.name, der.action, der.rate, der.to))
-            if der.rate == "infty":
+            if der.rate in ("infty", "T"):
                 der.rate = -1
             self.derivatives.append(Derivative(self.name, [der.to], der.action, der.rate, self.offset, arates[der.action], aprates=arates))
 
-
-
-    def __init__(self, ss, name,offset):
-        self.name = name
-        self.ss = ss
-        self.offset = offset
-        self.derivatives = []
-        arates = {}
-        for der in self.ss[self.name].transitions:
-            if der.action not in arates:
-                if der.rate == "infty":
-                    arates[der.action] = float(-1)
-                else:
-                    arates[der.action] = float(der.rate)
-            else:
-                arates[der.action] += float(der.rate)
-
-        for der in self.ss[self.name].transitions:
-            if der.rate == "infty":
-                der.rate = -1
-            self.derivatives.append(Derivative(self.name, [der.to], der.action, der.rate, self.offset, der.rate, aprates=arates))
-
     def get_derivatives(self):
         return self.derivatives
 
@@ -180,20 +220,22 @@ class Derivative():
         self.offset = offset
         self.combined = False
         self.arate = arate
-        self.apparent_rates = aprates
+        if aprates is not None:
+            self.apparent_rates = aprates 
+        else:
+            self.appparent_rates = {}
 
     def __str__(self):
         return " T:" + str(self.to_s) \
                 + " Act:"+self.action+" R:"+self.rate+" O:"+str(self.offset)+" Sh:"+str(self.shared)
 
-class Operator(Component):
-    length = None
-    offset = 0
-    actionset = []
-    lhs = None
-    rhs = None
-
+class Operator():
+    
     def __init__(self):
+        self.offset = 0
+        self.length = None
+        self.lhs = None
+        self.rhs = None
         self.actionset = []
         self.derivatives = []
         self.log = init_log()
@@ -220,8 +262,7 @@ class Operator(Component):
         lR = float(left) * float(right)
         new_rate = lR * float(self._min(tran_r.arate, tran_l.arate))
         # print("%s %s %s %s"% (left, right, lR, new_rate))
-        ddd = Derivative(state, to_state,tran_l.action,new_rate,self.offset, new_rate,True)
-        return ddd
+        return  Derivative(state, to_state,tran_l.action,new_rate,self.offset, new_rate,True, {})
 
     def _create_left_transition(self, state, tran_l):
         new_state = state[:]
@@ -286,6 +327,7 @@ class Operator(Component):
 
 
     def _min2(self,rate1, rate2):
+        """ Deprecated """
         if rate1 == "infty":
             if rate2 == "infty":
                 return "infty"
